@@ -24,17 +24,37 @@ import myokit
 #define PY_SSIZE_T_CLEAN
 #include <Python.h>
 #include <stdio.h>
+
 #include <cvodes/cvodes.h>
 #include <nvector/nvector_serial.h>
-#define MYOKIT_SUNDIALS_VERSION <?= myokit.SUNDIALS_VERSION ?>
-#if MYOKIT_SUNDIALS_VERSION >= 30000
+#include <sundials/sundials_types.h>
+#include <sundials/sundials_config.h>
+#ifndef SUNDIALS_VERSION_MAJOR
+    #define SUNDIALS_VERSION_MAJOR 2
+#endif
+#if SUNDIALS_VERSION_MAJOR >= 3
     #include <sunmatrix/sunmatrix_dense.h>
     #include <sunlinsol/sunlinsol_dense.h>
     #include <cvodes/cvodes_direct.h>
 #else
     #include <cvodes/cvodes_dense.h>
 #endif
-#include <sundials/sundials_types.h>
+
+<?
+if myokit.DEBUG_SM:
+    print('// Show debug output')
+    print('#ifndef MYOKIT_DEBUG_MESSAGES')
+    print('#define MYOKIT_DEBUG_MESSAGES')
+    print('#endif')
+
+# Note: When adding profiling messages, write them in past tense so that we can
+# show time elapsed for an operation **that has just completed**.
+if myokit.DEBUG_SP:
+    print('// Show profiling messages')
+    print('#ifndef MYOKIT_DEBUG_PROFILING')
+    print('#define MYOKIT_DEBUG_PROFILING')
+    print('#endif')
+?>
 
 #include "pacing.h"
 
@@ -192,10 +212,13 @@ double pace = 0;        /* Pacing value */
  * CVODE Memory
  */
 void *cvode_mem;     /* The memory used by the solver */
-#if MYOKIT_SUNDIALS_VERSION >= 30000
+#if SUNDIALS_VERSION_MAJOR >= 3
 SUNMatrix sundense_matrix;          /* Dense matrix for linear solves */
 SUNLinearSolver sundense_solver;    /* Linear solver object */
 #endif
+#if SUNDIALS_VERSION_MAJOR >= 6
+SUNContext sundials_context; /* A sundials context to run in (for profiling etc.) */
+#endif
 
 UserData udata;      /* UserData struct, used to pass in parameters */
 realtype* pbar;      /* Vector of independents in user data */
@@ -212,7 +235,6 @@ double dt_min = 0;      /* The minimum step size (0.0 for none) */
  * Solver stats
  */
 double realtime = 0;        /* Time since start */
-double realtime_start = 0;  /* Timestamp of sim init */
 long evaluations = 0;       /* Number of evaluations since sim init */
 long steps = 0;             /* Number of steps since sim init */
 
@@ -280,9 +302,44 @@ PyObject* rf_list;      /* List to store found roots in (or None if not enabled)
 int* rf_direction;      /* Direction of root crossings: 1 for up, -1 for down, 0 for no crossing. */
 
 /*
- * Benchmarking
+ * Logging realtime and profiling
  */
-PyObject* benchtime;    /* Callable time() function or None */
+PyObject* benchmarker;      /* myokit.tools.Benchmarker object */
+PyObject* benchmarker_time_str;
+int log_realtime;           /* 1 iff we're logging real simulation time */
+double realtime_start;      /* time when sim run started */
+
+/*
+ * Returns the current time as given by the benchmarker.
+ */
+double
+benchmarker_realtime()
+{
+    double val;
+    PyObject* ret = PyObject_CallMethodObjArgs(benchmarker, benchmarker_time_str, NULL);
+    if (!PyFloat_Check(ret)) {
+        Py_XDECREF(ret);
+        return -1.0;
+    }
+    val = PyFloat_AsDouble(ret);
+    Py_DECREF(ret);
+    return val - realtime_start;
+}
+
+#ifdef MYOKIT_DEBUG_PROFILING
+PyObject* benchmarker_print_str;
+
+/*
+ * Prints a message to screen, preceded by the time in ms as given by the benchmarker.
+ */
+void
+benchmarker_print(char* message)
+{
+    PyObject* pymsg = PyUnicode_FromString(message);
+    PyObject_CallMethodObjArgs(benchmarker, benchmarker_print_str, pymsg, NULL);
+    Py_DECREF(pymsg);
+}
+#endif
 
 /*
  * Right-hand-side function of the model ODE
@@ -377,6 +434,11 @@ PyObject*
 sim_clean()
 {
     if (initialised) {
+        #ifdef MYOKIT_DEBUG_PROFILING
+        benchmarker_print("CP Entered sim_clean.");
+        #elif defined MYOKIT_DEBUG_MESSAGES
+        printf("CM Cleaning up.\n");
+        #endif
 
         /* CVode arrays */
         if (y != NULL) { N_VDestroy_Serial(y); y = NULL; }
@@ -389,12 +451,15 @@ sim_clean()
         /* Root finding results */
         free(rf_direction); rf_direction = NULL;
 
-        /* CVode objects */
+        /* Sundials objects */
         CVodeFree(&cvode_mem); cvode_mem = NULL;
-        #if MYOKIT_SUNDIALS_VERSION >= 30000
+        #if SUNDIALS_VERSION_MAJOR >= 3
         SUNLinSolFree(sundense_solver); sundense_solver = NULL;
         SUNMatDestroy(sundense_matrix); sundense_matrix = NULL;
         #endif
+        #if SUNDIALS_VERSION_MAJOR >= 6
+        SUNContext_Free(&sundials_context); sundials_context = NULL;
+        #endif
 
         /* User data and parameter scale array*/
         free(pbar);
@@ -410,6 +475,13 @@ sim_clean()
         /* CModel */
         Model_Destroy(model); model = NULL;
 
+        /* Benchmarking and profiling */
+        #ifdef MYOKIT_DEBUG_PROFILING
+        benchmarker_print("CP Completed sim_clean.");
+        Py_XDECREF(benchmarker_print_str); benchmarker_print_str = NULL;
+        #endif
+        Py_XDECREF(benchmarker_time_str); benchmarker_time_str = NULL;
+
         /* Deinitialisation complete */
         initialised = 0;
     }
@@ -455,6 +527,10 @@ py_sim_clean(PyObject *self, PyObject *args)
 PyObject*
 sim_init(PyObject *self, PyObject *args)
 {
+    #ifdef MYOKIT_DEBUG_MESSAGES
+    printf("CM Entering sim_init.\n");
+    #endif
+
     /* Error checking flags */
     int flag_cvode;
     Model_Flag flag_model;
@@ -500,16 +576,24 @@ sim_init(PyObject *self, PyObject *args)
     ylast = NULL;
     /* Logging */
     log_times = NULL;
+    /* Benchmarking and profiling */
+    benchmarker_time_str = NULL;
+    #ifdef MYOKIT_DEBUG_PROFILING
+    benchmarker_print_str = NULL;
+    #endif
 
     /* CVode objects */
     cvode_mem = NULL;
-    #if MYOKIT_SUNDIALS_VERSION >= 30000
+    #if SUNDIALS_VERSION_MAJOR >= 3
     sundense_matrix = NULL;
     sundense_solver = NULL;
     #endif
+    #if SUNDIALS_VERSION_MAJOR >= 6
+    sundials_context = NULL;
+    #endif
 
     /* Check input arguments     0123456789012345678 */
-    if (!PyArg_ParseTuple(args, "ddOOOOOOOOdOOidOO",
+    if (!PyArg_ParseTuple(args, "ddOOOOOOOOdOOidOOi",
             &tmin,              /*  0. Float: initial time */
             &tmax,              /*  1. Float: final time */
             &state_py,          /*  2. List: initial and final state */
@@ -526,7 +610,8 @@ sim_init(PyObject *self, PyObject *args)
             &rf_indice,         /* 13. Int: root-finding state variable */
             &rf_threshold,      /* 14. Float: root-finding threshold */
             &rf_list,           /* 15. List to store roots in or None */
-            &benchtime          /* 16. Callable to obtain system time */
+            &benchmarker,       /* 16. myokit.tools.Benchmarker object */
+            &log_realtime       /* 17. Int: 1 if logging real time */
     )) {
         PyErr_SetString(PyExc_Exception, "Incorrect input arguments.");
         return 0;
@@ -588,28 +673,63 @@ sim_init(PyObject *self, PyObject *args)
     t = tmin;
 
     /* Reset solver stats */
-    evaluations = 0;
     steps = 0;
     zero_step_count = 0;
+    evaluations = 0;
+    realtime = 0;
+    if (log_realtime) {
+        realtime_start = 0; /* Updated after init, in first call to run */
+        benchmarker_time_str = PyUnicode_FromString("time");
+    }
+
+    /* Set up profiling */
+    #ifdef MYOKIT_DEBUG_PROFILING
+    benchmarker_print_str = PyUnicode_FromString("print");
+    benchmarker_print("CP Initialisation started (entered sim_init()).");
+    #endif
 
     /*
      * Create model
      */
     model = Model_Create(&flag_model);
     if (flag_model != Model_OK) { Model_SetPyErr(flag_model); return sim_clean(); }
+    #ifdef MYOKIT_DEBUG_PROFILING
+    benchmarker_print("CP Created C model struct.");
+    #endif
+
+    /*
+     * Create sundials context
+     */
+    #if SUNDIALS_VERSION_MAJOR >= 6
+    flag_cvode = SUNContext_Create(NULL, &sundials_context);
+    if (check_cvode_flag(&flag_cvode, "SUNContext_Create", 1)) {
+        return sim_cleanx(PyExc_Exception, "Failed to create Sundials context.");
+    }
+    #ifdef MYOKIT_DEBUG_PROFILING
+    benchmarker_print("CP Created sundials context.");
+    #endif
+    #endif
 
     /*
      * Create state vectors
      */
 
     /* Create state vector */
+    #if SUNDIALS_VERSION_MAJOR >= 6
+    y = N_VNew_Serial(model->n_states, sundials_context);
+    #else
     y = N_VNew_Serial(model->n_states);
+    #endif
     if (check_cvode_flag((void*)y, "N_VNew_Serial", 0)) {
         return sim_cleanx(PyExc_Exception, "Failed to create state vector.");
     }
 
     /* Create state vector copy for error handling */
+    #if SUNDIALS_VERSION_MAJOR >= 6
+    ylast = N_VNew_Serial(model->n_states, sundials_context);
+    #else
     ylast = N_VNew_Serial(model->n_states);
+    #endif
     if (check_cvode_flag((void*)ylast, "N_VNew_Serial", 0)) {
         return sim_cleanx(PyExc_Exception, "Failed to create last-state vector.");
     }
@@ -637,7 +757,11 @@ sim_init(PyObject *self, PyObject *args)
         z = y;
         sz = sy;
     } else {
+        #if SUNDIALS_VERSION_MAJOR >= 6
+        z = N_VNew_Serial(model->n_states, sundials_context);
+        #else
         z = N_VNew_Serial(model->n_states);
+        #endif
         if (check_cvode_flag((void*)z, "N_VNew_Serial", 0)) {
             return sim_cleanx(PyExc_Exception, "Failed to create state vector for logging.");
         }
@@ -649,6 +773,10 @@ sim_init(PyObject *self, PyObject *args)
         }
     }
 
+    #ifdef MYOKIT_DEBUG_PROFILING
+    benchmarker_print("CP Created sundials state vectors.");
+    #endif
+
     /*
      * Set initial state in model and vectors
      */
@@ -660,7 +788,7 @@ sim_init(PyObject *self, PyObject *args)
     for (i=0; i<model->n_states; i++) {
         val = PyList_GetItem(state_py, i);    /* Don't decref! */
         if (!PyFloat_Check(val)) {
-            return sim_cleanx(PyExc_TypeError, "Item %d in state vector is not a float.", i);
+            return sim_cleanx(PyExc_ValueError, "Item %d in state vector is not a float.", i);
         }
         model->states[i] = PyFloat_AsDouble(val);
         NV_Ith_S(y, i) = model->states[i];
@@ -674,12 +802,12 @@ sim_init(PyObject *self, PyObject *args)
         for (i=0; i<model->ns_independents; i++) {
             val = PyList_GetItem(s_state_py, i); /* Don't decref */
             if (!PyList_Check(val)) {
-                return sim_cleanx(PyExc_TypeError, "Item %d in state sensitivity matrix is not a list.", i);
+                return sim_cleanx(PyExc_ValueError, "Item %d in state sensitivity matrix is not a list.", i);
             }
             for (j=0; j<model->n_states; j++) {
                 ret = PyList_GetItem(val, j);    /* Don't decref! */
                 if (!PyFloat_Check(ret)) {
-                    return sim_cleanx(PyExc_TypeError, "Item %d, %d in state sensitivity matrix is not a float.", i, j);
+                    return sim_cleanx(PyExc_ValueError, "Item %d, %d in state sensitivity matrix is not a float.", i, j);
                 }
                 NV_Ith_S(sy[i], j) = PyFloat_AsDouble(ret);
                 model->s_states[i * model->n_states + j] = NV_Ith_S(sy[i], j);
@@ -687,6 +815,10 @@ sim_init(PyObject *self, PyObject *args)
         }
     }
 
+    #ifdef MYOKIT_DEBUG_PROFILING
+    benchmarker_print("CP Set initial state.");
+    #endif
+
     /*
      * Set values of constants (literals and parameters)
      */
@@ -696,7 +828,7 @@ sim_init(PyObject *self, PyObject *args)
     for (i=0; i<model->n_literals; i++) {
         val = PyList_GetItem(literals, i);    /* Don't decref */
         if (!PyFloat_Check(val)) {
-            return sim_cleanx(PyExc_TypeError, "Item %d in literal vector is not a float.", i);
+            return sim_cleanx(PyExc_ValueError, "Item %d in literal vector is not a float.", i);
         }
         model->literals[i] = PyFloat_AsDouble(val);
     }
@@ -704,6 +836,10 @@ sim_init(PyObject *self, PyObject *args)
     /* Evaluate calculated constants */
     Model_EvaluateLiteralDerivedVariables(model);
 
+    #ifdef MYOKIT_DEBUG_PROFILING
+    benchmarker_print("CP Set model constants and calculated derived constants.");
+    #endif
+
     /* Set model parameters */
     if (model->has_sensitivities) {
         if (!PyList_Check(parameters)) {
@@ -712,13 +848,17 @@ sim_init(PyObject *self, PyObject *args)
         for (i=0; i<model->n_parameters; i++) {
             val = PyList_GetItem(parameters, i);    /* Don't decref */
             if (!PyFloat_Check(val)) {
-                return sim_cleanx(PyExc_TypeError, "Item %d in parameter vector is not a float.", i);
+                return sim_cleanx(PyExc_ValueError, "Item %d in parameter vector is not a float.", i);
             }
             model->parameters[i] = PyFloat_AsDouble(val);
         }
 
         /* Evaluate calculated constants */
         Model_EvaluateParameterDerivedVariables(model);
+
+        #ifdef MYOKIT_DEBUG_PROFILING
+        benchmarker_print("CP Setting model sensitivity parameters and calculated derived quantities.");
+        #endif
     }
 
     /* Create UserData with sensitivity vector */
@@ -750,6 +890,10 @@ sim_init(PyObject *self, PyObject *args)
         for (i=0; i<model->ns_independents; i++) {
             pbar[i] = (udata->p[i] == 0.0 ? 1.0 : fabs(udata->p[i]));
         }
+
+        #ifdef MYOKIT_DEBUG_PROFILING
+        benchmarker_print("CP Created UserData for sensitivities.");
+        #endif
     }
 
     /*
@@ -767,6 +911,10 @@ sim_init(PyObject *self, PyObject *args)
         tnext = ESys_GetNextTime(epacing, &flag_epacing);
         pace = ESys_GetLevel(epacing, &flag_epacing);
         tnext = (tnext < tmax) ? tnext : tmax;
+
+        #ifdef MYOKIT_DEBUG_PROFILING
+        benchmarker_print("CP Created event-based pacing system.");
+        #endif
     } else {
         tnext = tmax;
     }
@@ -787,6 +935,10 @@ sim_init(PyObject *self, PyObject *args)
             PyTuple_GetItem(fprotocol, 0),  /* Borrowed, no decref */
             PyTuple_GetItem(fprotocol, 1));
         if (flag_fpacing != FSys_OK) { FSys_SetPyErr(flag_fpacing); return sim_clean(); }
+
+        #ifdef MYOKIT_DEBUG_PROFILING
+        benchmarker_print("CP Created fixed-form pacing system.");
+        #endif
     }
 
     /*
@@ -795,8 +947,10 @@ sim_init(PyObject *self, PyObject *args)
     if (model->is_ode) {
 
         /* Create, using backwards differentiation and newton iterations */
-        #if MYOKIT_SUNDIALS_VERSION >= 40000
-        cvode_mem = CVodeCreate(CV_BDF); /* Newton is still default */
+        #if SUNDIALS_VERSION_MAJOR >= 6
+        cvode_mem = CVodeCreate(CV_BDF, sundials_context);
+        #elif SUNDIALS_VERSION_MAJOR >= 4
+        cvode_mem = CVodeCreate(CV_BDF);  /* Newton is still default */
         #else
         cvode_mem = CVodeCreate(CV_BDF, CV_NEWTON);
         #endif
@@ -822,7 +976,19 @@ sim_init(PyObject *self, PyObject *args)
         flag_cvode = CVodeSetMinStep(cvode_mem, dt_min < 0 ? 0.0 : dt_min);
         if (check_cvode_flag(&flag_cvode, "CVodeSetminStep", 1)) return sim_clean();
 
-        #if MYOKIT_SUNDIALS_VERSION >= 40000
+        #if SUNDIALS_VERSION_MAJOR >= 6
+            /* Create dense matrix for use in linear solves */
+            sundense_matrix = SUNDenseMatrix(model->n_states, model->n_states, sundials_context);
+            if (check_cvode_flag((void *)sundense_matrix, "SUNDenseMatrix", 0)) return sim_clean();
+
+            /* Create dense linear solver object with matrix */
+            sundense_solver = SUNLinSol_Dense(y, sundense_matrix, sundials_context);
+            if (check_cvode_flag((void *)sundense_solver, "SUNLinSol_Dense", 0)) return sim_clean();
+
+            /* Attach the matrix and solver to cvode */
+            flag_cvode = CVodeSetLinearSolver(cvode_mem, sundense_solver, sundense_matrix);
+            if (check_cvode_flag(&flag_cvode, "CVodeSetLinearSolver", 1)) return sim_clean();
+        #elif SUNDIALS_VERSION_MAJOR >= 4
             /* Create dense matrix for use in linear solves */
             sundense_matrix = SUNDenseMatrix(model->n_states, model->n_states);
             if (check_cvode_flag((void *)sundense_matrix, "SUNDenseMatrix", 0)) return sim_clean();
@@ -834,7 +1000,7 @@ sim_init(PyObject *self, PyObject *args)
             /* Attach the matrix and solver to cvode */
             flag_cvode = CVodeSetLinearSolver(cvode_mem, sundense_solver, sundense_matrix);
             if (check_cvode_flag(&flag_cvode, "CVodeSetLinearSolver", 1)) return sim_clean();
-        #elif MYOKIT_SUNDIALS_VERSION >= 30000
+        #elif SUNDIALS_VERSION_MAJOR >= 3
             /* Create dense matrix for use in linear solves */
             sundense_matrix = SUNDenseMatrix(model->n_states, model->n_states);
             if (check_cvode_flag((void *)sundense_matrix, "SUNDenseMatrix", 0)) return sim_clean();
@@ -852,6 +1018,10 @@ sim_init(PyObject *self, PyObject *args)
             if (check_cvode_flag(&flag_cvode, "CVDense", 1)) return sim_clean();
         #endif
 
+        #ifdef MYOKIT_DEBUG_PROFILING
+        benchmarker_print("CP CVODES solver initialised.");
+        #endif
+
         /* Activate forward sensitivity computations */
         if (model->has_sensitivities) {
             /* TODO: NULL here is the place to insert a user function to calculate the
@@ -871,6 +1041,10 @@ sim_init(PyObject *self, PyObject *args)
             /* Set sensitivity tolerances calculating method (using pbar) */
             flag_cvode = CVodeSensEEtolerances(cvode_mem);
             if (check_cvode_flag(&flag_cvode, "CVodeSensEEtolerances", 1)) return sim_clean();
+
+            #ifdef MYOKIT_DEBUG_PROFILING
+            benchmarker_print("CP CVODES sensitivity methods initialised.");
+            #endif
         }
     }
 
@@ -887,16 +1061,10 @@ sim_init(PyObject *self, PyObject *args)
 
         /* Direction of root crossings, one entry per root function, but we only use 1. */
         rf_direction = (int*)malloc(sizeof(int)*1);
-    }
 
-    /*
-     * Benchmarking and solver stats
-     */
-    if (benchtime != Py_None) {
-        /* Store initial time as 0 */
-        realtime = 0.0;
-        /* Tell sim_step to set realtime_start */
-        realtime_start = -1;
+        #ifdef MYOKIT_DEBUG_PROFILING
+        benchmarker_print("CP CVODES root-finding initialised.");
+        #endif
     }
 
     /*
@@ -913,6 +1081,9 @@ sim_init(PyObject *self, PyObject *args)
     /* Set up logging */
     flag_model = Model_InitialiseLogging(model, log_dict);
     if (flag_model != Model_OK) { Model_SetPyErr(flag_model); return sim_clean(); }
+    #ifdef MYOKIT_DEBUG_PROFILING
+    benchmarker_print("CP Logging initialised.");
+    #endif
 
     /* Check logging list for sensitivities */
     if (model->has_sensitivities) {
@@ -920,6 +1091,7 @@ sim_init(PyObject *self, PyObject *args)
             return sim_cleanx(PyExc_TypeError, "'sens_list' must be a list.");
         }
     }
+
     /* Set logging points */
     if (log_interval > 0) {
 
@@ -931,22 +1103,31 @@ sim_init(PyObject *self, PyObject *args)
 
         /* Point-list logging */
 
-        /* Check the log_times list */
-        if (!PyList_Check(log_times)) {
-            return sim_cleanx(PyExc_TypeError, "'log_times' must be a list.");
+        /* Check the log_times sequence */
+        if (!PySequence_Check(log_times)) {
+            return sim_cleanx(PyExc_TypeError, "'log_times' must be a sequence type.");
         }
 
-        /* Read next log point off the list */
+        /* Read next log point off the sequence */
         ilog = 0;
         tlog = t - 1;
-        while(ilog < PyList_Size(log_times) && tlog < t) {
-            val = PyList_GetItem(log_times, ilog); /* Borrowed */
-            if (!PyFloat_Check(val)) {
-                return sim_cleanx(PyExc_TypeError, "Entries in 'log_times' must be floats.");
+        while(ilog < PySequence_Size(log_times) && tlog < t) {
+            val = PySequence_GetItem(log_times, ilog); /* Borrowed */
+            if (PyFloat_Check(val)) {
+                tlog = PyFloat_AsDouble(val);
+            } else if (PyNumber_Check(val)) {
+                val = PyNumber_Float(val);
+                if (val == NULL) {
+                    return sim_cleanx(PyExc_ValueError, "Unable to cast entry in 'log_times' to float.");
+                } else {
+                    tlog = PyFloat_AsDouble(val);
+                    Py_DECREF(val);
+                }
+            } else {
+                return sim_cleanx(PyExc_ValueError, "Entries in 'log_times' must be floats.");
             }
-            tlog = PyFloat_AsDouble(val);
-            ilog++;
             val = NULL;
+            ilog++;
         }
 
         /* No points beyond time? Then don't log any future points. */
@@ -994,9 +1175,16 @@ sim_init(PyObject *self, PyObject *args)
         }
     }
 
+    #ifdef MYOKIT_DEBUG_PROFILING
+    benchmarker_print("CP Logging times and strategy initialised.");
+    #endif
+
     /*
      * Done!
      */
+    #ifdef MYOKIT_DEBUG_PROFILING
+    benchmarker_print("CP Initialisation complete (returning from sim_init).");
+    #endif
     Py_RETURN_NONE;
 }
 
@@ -1019,22 +1207,22 @@ sim_step(PyObject *self, PyObject *args)
     /* Number of integration steps taken in this call */
     int steps_taken = 0;
 
+    /* Proposed next logging point */
+    double proposed_tlog;
+
     /* Multi-purpose Python object */
     PyObject *val;
 
     /*
-     * Benchmarking? Then make sure start time is set.
+     * Set start time for logging of realtime.
      * This is handled here instead of in sim_init so it only includes time
      * taken performing steps, not time initialising memory etc.
      */
-    if (benchtime != Py_None && realtime_start < 0) {
-        val = PyObject_CallFunction(benchtime, "");
-        if (!PyFloat_Check(val)) {
-            Py_XDECREF(val); val = NULL;
-            return sim_cleanx(PyExc_TypeError, "Call to benchmark time function didn't return float.");
+    if (log_realtime && realtime_start == 0) {
+        realtime_start = benchmarker_realtime();
+        if (realtime_start <= 0) {
+            return sim_cleanx(PyExc_Exception, "Failed to set realtime_start.");
         }
-        realtime_start = PyFloat_AsDouble(val);
-        Py_DECREF(val); val = NULL;
     }
 
     /* Go! */
@@ -1051,8 +1239,8 @@ sim_step(PyObject *self, PyObject *args)
         if (model->is_ode) {
 
             /* Take a single ODE step */
-            #ifdef MYOKIT_DEBUG
-            printf("\nTaking CVODE step from time %f to %f\n", t, tnext);
+            #ifdef MYOKIT_DEBUG_MESSAGES
+            printf("\nCM Taking CVODE step from time %f to %f\n", t, tnext);
             #endif
             flag_cvode = CVode(cvode_mem, tnext, y, &t, CV_ONE_STEP);
 
@@ -1103,8 +1291,8 @@ sim_step(PyObject *self, PyObject *args)
 
                 /* Next event time exceeded? */
                 if (t > tnext) {
-                    #ifdef MYOKIT_DEBUG
-                    printf("Event time exceeded, rewinding to %f\n", tnext);
+                    #ifdef MYOKIT_DEBUG_MESSAGES
+                    printf("CM Event time exceeded, rewinding to %f\n", tnext);
                     #endif
 
                     /* Go back to time=tnext */
@@ -1158,19 +1346,14 @@ sim_step(PyObject *self, PyObject *args)
 
                 /* Log points */
                 while (t > tlog) {
-                    #ifdef MYOKIT_DEBUG
-                    printf("Interpolation-logging for t=%f\n", t);
+                    #ifdef MYOKIT_DEBUG_MESSAGES
+                    printf("CM Interpolation-logging for t=%f\n", t);
                     #endif
 
                     /* Benchmarking? Then set realtime */
-                    if (benchtime != Py_None) {
-                        val = PyObject_CallFunction(benchtime, "");
-                        if (!PyFloat_Check(val)) {
-                            Py_XDECREF(val);
-                            return sim_cleanx(PyExc_TypeError, "Call to benchmark function did not return float.");
-                        }
-                        realtime = PyFloat_AsDouble(val) - realtime_start;
-                        Py_DECREF(val); val = NULL;
+                    if (log_realtime) {
+                        realtime = benchmarker_realtime();
+                        if (realtime < 0) return sim_cleanx(PyExc_Exception, "Failed to set realtime during interpolation logging.");
                     }
 
                     /* Get interpolated y(tlog) */
@@ -1212,13 +1395,26 @@ sim_step(PyObject *self, PyObject *args)
                         }
                     } else {
                         /* Point-list logging */
-                        /* Read next log point off the list */
-                        if (ilog < PyList_Size(log_times)) {
-                            val = PyList_GetItem(log_times, ilog); /* Borrowed */
-                            if (!PyFloat_Check(val)) {
+                        /* Read next log point off the sequence */
+                        if (ilog < PySequence_Size(log_times)) {
+                            val = PySequence_GetItem(log_times, ilog); /* Borrowed */
+                            if (PyFloat_Check(val)) {
+                                proposed_tlog = PyFloat_AsDouble(val);
+                            } else if (PyNumber_Check(val)) {
+                                val = PyNumber_Float(val);  /* New reference */
+                                if (val == NULL) {
+                                    return sim_cleanx(PyExc_ValueError, "Unable to cast entry in 'log_times' to float.");
+                                } else {
+                                    proposed_tlog = PyFloat_AsDouble(val);
+                                    Py_DECREF(val);
+                                }
+                            } else {
                                 return sim_cleanx(PyExc_ValueError, "Entries in 'log_times' must be floats.");
                             }
-                            tlog = PyFloat_AsDouble(val);
+                            if (proposed_tlog < tlog) {
+                                return sim_cleanx(PyExc_ValueError, "Values in log_times must be non-decreasing.");
+                            }
+                            tlog = proposed_tlog;
                             ilog++;
                             val = NULL;
                         } else {
@@ -1248,14 +1444,9 @@ sim_step(PyObject *self, PyObject *args)
             if (dynamic_logging) {
 
                 /* Benchmarking? Then set realtime */
-                if (benchtime != Py_None) {
-                    val = PyObject_CallFunction(benchtime, "");
-                    if (!PyFloat_Check(val)) {
-                        Py_XDECREF(val);
-                        return sim_cleanx(PyExc_TypeError, "Call to benchmark time function did not return float.");
-                    }
-                    realtime = PyFloat_AsDouble(val) - realtime_start;
-                    Py_DECREF(val); val = NULL;
+                if (log_realtime) {
+                    realtime = benchmarker_realtime();
+                    if (realtime < 0) return sim_cleanx(PyExc_Exception, "Failed to set realtime during dynamic logging.");
                 }
 
                 /* Ensure the logged values are correct for the new time t */
@@ -1263,16 +1454,14 @@ sim_step(PyObject *self, PyObject *args)
                     /* If logging derivatives or intermediaries, calculate the
                        values for the current time. Similarly, if calculating
                        sensitivities this is needed. */
-                    #ifdef MYOKIT_DEBUG
-                    printf("Calling RHS to log derivs/inter/sens at time %f\n", t);
+                    #ifdef MYOKIT_DEBUG_MESSAGES
+                    printf("CM Calling RHS to log derivs/inter/sens at time %f\n", t);
                     #endif
                     rhs(t, y, NULL, udata);
                 } else if (model->logging_bound) {
                     /* Logging bounds but not derivs or inters: No need to run
                        full rhs, just update bound variables */
-                    Model_SetBoundVariables(
-                        model, t, pace,
-                        realtime, evaluations);
+                    Model_SetBoundVariables(model, t, pace, realtime, evaluations);
                 }
 
                 /* Write to log */
@@ -1323,9 +1512,15 @@ sim_step(PyObject *self, PyObject *args)
          */
         steps_taken++;
         if (steps_taken >= 100) {
+            #ifdef MYOKIT_DEBUG_PROFILING
+            benchmarker_print("CP Completed 100 steps, passing control back to Python.");
+            #endif
             return PyFloat_FromDouble(t);
         }
     }
+    #ifdef MYOKIT_DEBUG_PROFILING
+    benchmarker_print("CP Completed remaining simulation steps.");
+    #endif
 
     /*
      * Finished! Set final state
@@ -1353,6 +1548,10 @@ sim_step(PyObject *self, PyObject *args)
     PyList_SetItem(bound_py, 2, PyFloat_FromDouble(realtime));
     PyList_SetItem(bound_py, 3, PyFloat_FromDouble(evaluations));
 
+    #ifdef MYOKIT_DEBUG_PROFILING
+    benchmarker_print("CP Set final state and bound variable values.");
+    #endif
+
     sim_clean();    /* Ignore return value */
     return PyFloat_FromDouble(t);
 }
