@@ -300,18 +300,11 @@ class SimulationOpenCL(myokit.CModule):
             'dims': len(self._dims),
         }
 
-        # Debug
-        if myokit.DEBUG:
-            print(self._code(
-                fname, args, line_numbers=myokit.DEBUG_LINE_NUMBERS))
-            #import sys
-            #sys.exit(1)
-
         # Define libraries
         libs = []
         flags = []
         plat = platform.system()
-        if plat != 'Darwin':    # pragma: no osx cover
+        if plat != 'Darwin':    # pragma: no macos cover
             libs.append('OpenCL')
         else:                   # pragma: no cover
             flags.append('-framework')
@@ -324,7 +317,8 @@ class SimulationOpenCL(myokit.CModule):
         incd = list(myokit.OPENCL_INC)
         incd.append(myokit.DIR_CFUNC)
         self._sim = self._compile(
-            mname, fname, args, libs, libd, incd, larg=flags)
+            mname, fname, args, libs, libd, incd, larg=flags,
+            continue_in_debug_mode=True)
 
     def calculate_conductance(self, r, sx, chi, dx):
         """
@@ -1057,13 +1051,6 @@ class SimulationOpenCL(myokit.CModule):
             'heterogeneous': self._gx_field is not None,
             'fiber_tissue': False,
         }
-        if myokit.DEBUG:    # pragma: no cover
-            print('-' * 79)
-            print(
-                self._code(kernel_file, args,
-                           line_numbers=myokit.DEBUG_LINE_NUMBERS))
-            import sys
-            sys.exit(1)
         kernel = self._export(kernel_file, args)
 
         # Logging period (0 = disabled)
