@@ -8,6 +8,7 @@ from __future__ import absolute_import, division
 from __future__ import print_function, unicode_literals
 
 import collections
+import warnings
 
 from lxml import etree
 
@@ -16,7 +17,7 @@ import myokit.formats.mathml
 import myokit.formats.cellml as cellml
 import myokit.formats.cellml.v1
 
-from myokit.mxml import split
+from myokit.formats.xml import split
 
 
 def parse_file(path):
@@ -709,18 +710,20 @@ class CellMLParser(object):
             # Find units in component
             try:
                 units = component.find_units(units)
-            except myokit.formats.cellml.v1.UnsupportedUnitsError:
-                raise CellMLParsingError(
-                    'Unsupported unit "' + str(units) + '" referenced inside a'
-                    ' MathML equation (4.4.3.2).', element)
-
+                units = units.myokit_unit()
+            except myokit.formats.cellml.v1.UnitsError as e:
+                warnings.warn(
+                    'The units "' + str(units) + '" (referenced inside a'
+                    ' MathML equation) are not supported and have been'
+                    ' replaced by `dimensionless`. (' + str(e) + ')')
+                units = myokit.units.dimensionless
             except myokit.formats.cellml.v1.CellMLError:
                 raise CellMLParsingError(
                     'Unknown unit "' + str(units) + '" referenced inside a'
                     ' MathML equation (4.4.3.2).', element)
 
             # Create and return
-            return myokit.Number(value, units.myokit_unit())
+            return myokit.Number(value, units)
 
         # Create parser
         p = myokit.formats.mathml.MathMLParser(
@@ -980,8 +983,7 @@ class CellMLParser(object):
             raise CellMLParsingError(
                 'Unit offset must be a real number (5.4.2.6).', element)
         if x != 0:
-            raise CellMLParsingError(
-                'Units with non-zero offsets are not supported.', element)
+            raise myokit.formats.cellml.v1.UnsupportedUnitOffsetError
 
         # Check allowed content
         self._check_allowed_content(
@@ -1005,10 +1007,7 @@ class CellMLParser(object):
 
         # Check if this is a base unit (not supported)
         base = element.attrib.get('base_units', 'no')
-        if base == 'yes':
-            raise CellMLParsingError(
-                'Defining new base units is not supported.', element)
-        elif base != 'no':
+        if base not in ('yes', 'no'):
             raise CellMLParsingError(
                 'Base units attribute must be either "yes" or "no".', element)
 
@@ -1022,14 +1021,26 @@ class CellMLParser(object):
         # Check the units definition has children
         children = element.findall(self._join('unit'))
         if not children:
-            raise CellMLParsingError(
-                'Units element with base_units="no" must contain at least one'
-                ' child unit element.')
+            if base == 'yes':
+                warnings.warn(
+                    'Unable to parse definition for units "' + str(name) + '",'
+                    ' using `dimensionless` instead. (Defining new base units'
+                    ' is not supported.)')
+            else:
+                raise CellMLParsingError(
+                    'Units element with base_units="no" must contain at least'
+                    ' one child unit element.')
 
         # Parse content
         myokit_unit = myokit.units.dimensionless
-        for child in children:
-            myokit_unit *= self._parse_unit(child, owner)
+        try:
+            for child in children:
+                myokit_unit *= self._parse_unit(child, owner)
+        except myokit.formats.cellml.v1.UnitsError as e:
+            warnings.warn(
+                'Unable to parse definition for units "' + str(name) + '",'
+                ' using `dimensionless` instead. (' + str(e) + ')')
+            myokit_unit = myokit.units.dimensionless
 
         # Add units to owner
         try:
@@ -1064,7 +1075,15 @@ class CellMLParser(object):
 
         # Create variable (validates name, units, and name uniqueness)
         try:
-            variable = component.add_variable(name, units, pub, pri)
+            try:
+                variable = component.add_variable(name, units, pub, pri)
+            except myokit.formats.cellml.v1.UnitsError as e:
+                warnings.warn(
+                    'The units "' + str(units) + '" (assigned to a variable)'
+                    ' are not supported and have been replaced by'
+                    ' `dimensionless`. (' + str(e) + ')')
+                units = 'dimensionless'
+                variable = component.add_variable(name, units, pub, pri)
 
             # Check cmeta id
             cmeta_id = self._check_cmeta_id(element)
