@@ -2,10 +2,8 @@
 #
 # ifthenelse.m :: Handles piecewise constructs in expressions
 #
-# This file is part of Myokit
-#  Copyright 2011-2018 Maastricht University, University of Oxford
-#  Licensed under the GNU General Public License v3.0
-#  See: http://myokit.org
+# This file is part of Myokit.
+# See http://myokit.org for copyright, sharing, and licensing details.
 #
 ?>% Emulate the ternary operator x = (cond) ? iftrue : iffalse
 function y = ifthenelse(cond, iftrue, iffalse)
